@@ -132,7 +132,7 @@ class CurricularUnit:
        
         contents = soup.find("div", {"id" : "conteudoinner"})
 
-        self.name = contents.find_all("h1")[1].string
+        self.name = contents.find_all("h1")[1].string.strip()
 
         first_table = contents.find("table")
         self._parse_first_table(first_table)
@@ -166,10 +166,10 @@ class CurricularUnit:
         if len(matches) == 1:
             self.semester = matches[0]
         else:
-            matches = _re.findall(r"Instance: \d\d\d\d/\d\d\d\d - A", str(contents))
+            matches = _re.findall(r"Instance: \d\d\d\d/\d\d\d\d - (\w+)", str(contents))
             
             if len(matches) == 1:
-                self.semester = 'A'
+                self.semester = matches[0]
             else:
                 self.semester = None
 
@@ -396,6 +396,9 @@ class CurricularUnit:
             ]
         """
 
+        if self.number_of_students == 0:
+            return []
+
         data   = []
         tables = []
 
@@ -414,7 +417,7 @@ class CurricularUnit:
 
             data.append((student, status, number_of_registrations, student_type))
                     
-        n_pages    = (self.number_of_students // 50) + 1
+        n_pages    = ((self.number_of_students - 1) // 50) + 1 # Example: ((200 - 1) // 50) + 1 = 4
         pages_urls = []
         for n in range(1, n_pages + 1):
             url = self.base_url + _utils.SIG_URLS["curricular unit students"] + "?" + _urllib.parse.urlencode({"pv_ocorrencia_id" : str(self.pv_ocorrencia_id), "pv_num_pag" : str(n)})
@@ -437,7 +440,7 @@ class CurricularUnit:
         
         return data
     
-    def timetable(self, credentials : _Credentials.Credentials) -> list:
+    def timetable(self, credentials : _Credentials.Credentials, ignore_coherence : bool = False) -> list:
         """Returns the curricular unit's current timetable as a list of dictionaries if possible, otherwise returns None.
         (see :func:`timetable.parse_current_timetable` for more info)
         
@@ -447,9 +450,9 @@ class CurricularUnit:
         html = credentials.get_html(self.base_url.replace("/en/", "/pt/") + _utils.SIG_URLS["curricular unit timetable"], {"pv_ocorrencia_id" : self.pv_ocorrencia_id})
         soup = _bs4.BeautifulSoup(html, "lxml")
 
-        return _timetable.parse_current_timetable(credentials, soup.a["href"])
-    
-    def all_timetables(self, credentials : _Credentials.Credentials) -> dict:
+        return _timetable.parse_current_timetable(credentials, soup.a["href"], ignore_coherence)
+
+    def all_timetables(self, credentials : _Credentials.Credentials, ignore_coherence : bool = False) -> dict:
         """Parses all the timetables related to this curricular unit
         (see :obj:`timetable.parse_timetables` for further info).
         
@@ -462,7 +465,7 @@ class CurricularUnit:
         html = credentials.get_html(self.base_url.replace("/en/", "/pt/") + _utils.SIG_URLS["curricular unit timetable"], {"pv_ocorrencia_id" : self.pv_ocorrencia_id})
         soup = _bs4.BeautifulSoup(html, "lxml")
 
-        return _timetable.parse_timetables(credentials, soup.a["href"])
+        return _timetable.parse_timetables(credentials, soup.a["href"], ignore_coherence)
 
     def other_occurrences(self, use_cache : bool = True) -> tuple:
         """Returns the occurrences of this curricular unit from other years as a
@@ -678,12 +681,14 @@ class CurricularUnit:
 
         return result
     
-    def classes(self, credentials : _Credentials.Credentials, use_cache : bool = True) -> dict:
+    def classes(self, credentials : _Credentials.Credentials, full_info = False, use_cache : bool = True) -> dict:
         """Returns a dictionary which maps a class name to a list of students
         (the students of that class).
         
         Args:
             credentials (:obj:`Credentials`): A :obj:`Credentials` object
+            full_info (:obj:`bool`, optional): Returns a list of tuples. Each tuple contains the student, the allocation date,
+                whether or not it was allocated by the administration, and whether or not the student is enrolled
             use_cache (:obj:`bool`, optional): Attempts to use the cache if True, otherwise it will fetch from sigarra
 
         Returns:
@@ -742,14 +747,38 @@ class CurricularUnit:
             while title.find_next("h3") != None:
                 title = title.find_next("h3") # Skip to the next class
 
-                class_name = _re.findall(r"Turma: ([^\s]+)", title.text)[0]
+                class_name = _re.findall(r"Class: ([^\s]+)", title.text)[0]
 
                 table = title.find_next("table")
 
                 if table == None:
                     students = []
                 else:
-                    students = [_Student.Student.from_a_tag(tag, base_url = self.base_url) for tag in table.find_all("a")]
+                    if full_info:
+                        students = []
+
+                        for row in table.find_all("tr")[1:]:
+                            collumns = row.find_all("td")
+
+                            student = _Student.Student.from_a_tag(collumns[0].next, base_url = self.base_url)
+
+                            alloc_date = _utils.parse_date(str(collumns[3]))
+
+                            allocation_by_admin = "Allocation by the administration" in str(collumns[3])
+
+                            if "Verificar" in str(collumns[4]):
+                                enrolled = True
+                            elif "Delete" in str(collumns[4]):
+                                enrolled = False
+                            else:
+                                raise Exception(f"Classes parsing went wrong while parsing {self}")
+
+                            students.append(
+                                (student, alloc_date, allocation_by_admin, enrolled)
+                            )
+
+                    else:
+                        students = [_Student.Student.from_a_tag(tag, base_url = self.base_url) for tag in table.find_all("a")]
 
                 result[class_name] = students
 
