@@ -18,14 +18,44 @@ _weekdays_pt = ("Segunda", "Terça", "Quarta",
 
 _weekdays_pt_to_en = {pt: en for pt, en in zip(_weekdays_pt, _weekdays)}
 
+class CoherenceError(Exception):
+    """In very rare circumstances, it may be impossible to fetch a lesson's data.
+    This exception class is here to deal with that possibility.
+    
+    Args:
+        url (str): The url of the timetable page that is giving problems
+        minute (int)
+        hour (int)
+        weekday (str): "Monday", "Tuesday", etc.
 
-def parse_current_timetable(credentials: _Credentials.Credentials, url: str):
+    Attributes:
+        url (str): The url of the timetable page that is giving problems
+        minute (int)
+        hour (int)
+        weekday (str): "Monday", "Tuesday", etc."""
+
+    def __init__(self, url, minute, hour, weekday):
+        self.url = url
+        self.minute = minute
+        self.hour = hour
+        self.weekday = weekday
+
+        super().__init__(
+           f"An event wasn't found in the following timetable link:\n{url}\n"
+           f"A lesson that starts at {hour}:{minute} at {weekday} doesn't appear to exist but it should\n"
+            "Note: If you see a \"Aulas Sobrepostas\" table at the bottom of the page (see link), this exception is expected.\n"
+            "Unfortunately, there is no known way to fetch this event's data :(\n"
+        )
+
+def parse_current_timetable(credentials: _Credentials.Credentials, url: str, ignore_coherence : bool = False):
     """Attempts to return the related timetable that is valid today as a list of dictionaries.
     If no timetable is valid today, it returns None.
     
     Args:
         credentials (:obj:`feupy.Credentials`): A :obj:`feupy.Credentials` object
         url (str): The url of the timetable page
+        ignore_coherence (:obj:`bool`, optional): Whether or not this function should raise a :obj:`CoherenceError` exception
+            if the information of a class can not be fetched
 
     Returns:
         A list of dicts (e.g. see :func:`parse_timetable`) or None
@@ -34,17 +64,19 @@ def parse_current_timetable(credentials: _Credentials.Credentials, url: str):
     
     for (start, finish), url in _parse_side_bar(credentials, url).items():
         if start <= _datetime.date.today() <= finish:
-            return parse_timetable(credentials, url)
+            return parse_timetable(credentials, url, ignore_coherence)
     else:
         return None
 
 
-def parse_timetables(credentials: _Credentials.Credentials, url: str) -> dict:
+def parse_timetables(credentials: _Credentials.Credentials, url: str, ignore_coherence : bool = False) -> dict:
     """Returns the timetables related to this timetable (including) as a dictionary.
 
     Args:
         credentials (:obj:`feupy.Credentials`): A :obj:`feupy.Credentials` object
         url (str): The url of the timetable page
+        ignore_coherence (:obj:`bool`, optional): Whether or not this function should raise a :obj:`CoherenceError` exception
+            if the information of a class can not be fetched
 
     Returns:
         A dictionary which maps a tuple with two :obj:`datetime.date` objects,
@@ -231,7 +263,7 @@ def parse_timetables(credentials: _Credentials.Credentials, url: str) -> dict:
     result = _parse_side_bar(credentials, url)
 
     for key in result:
-        result[key] = parse_timetable(credentials, result[key])
+        result[key] = parse_timetable(credentials, result[key], ignore_coherence)
 
     return result
 
@@ -266,7 +298,7 @@ def _parse_side_bar(credentials: _Credentials.Credentials, url: str) -> dict:
         result[_parse_dates(tag.string, academic_year)] = credentials.base_url.replace("/en/", "/pt/") + tag["href"]
     return result
 
-def parse_timetable(credentials: _Credentials.Credentials, url: str) -> list:
+def parse_timetable(credentials: _Credentials.Credentials, url: str, ignore_coherence : bool = False) -> list:
     """Parses the events (including overlaps) of the timetable
     with the given url as a list of dictionaries.
     
@@ -288,6 +320,8 @@ def parse_timetable(credentials: _Credentials.Credentials, url: str) -> list:
     Args:
         credentials (:obj:`feupy.Credentials`): A :obj:`feupy.Credentials` object
         url (str): The url of the timetable page
+        ignore_coherence (:obj:`bool`, optional): Whether or not this function should raise a :obj:`CoherenceError` exception
+            if the information of a class can not be fetched
 
     Returns:
         A list of dicts
@@ -366,8 +400,8 @@ def parse_timetable(credentials: _Credentials.Credentials, url: str) -> list:
         ... ]
     """
     try:
-        weeks_url_query = re.findall(r"&p_semana_inicio=\d+&p_semana_fim=\d+$", url)[0]
-    except:
+        weeks_url_query = _re.findall(r"&p_semana_inicio=\d+&p_semana_fim=\d+$", url)[0]
+    except IndexError:
         weeks_url_query = ""
 
     html = credentials.get_html(url)
@@ -421,7 +455,12 @@ def parse_timetable(credentials: _Credentials.Credentials, url: str) -> list:
 
     last_table = timetable_soup.find_next("table", {"class": "dados"})
 
-    if "Aulas Sobrepostas" in str(last_table):
+    if "pv_turma_id" in url:
+        pass 
+        # There is a weird-ass bug that shows events that are perfectly fine as overlapping, which causes runaway recursion and lots of tears
+        # See https://sigarra.up.pt/feup/pt/hor_geral.turmas_view?pv_ano_lectivo=2019&pv_periodos=2&pv_turma_id=209033&p_semana_inicio=20190922&p_semana_fim=20191012
+        # With "pv_turma_id" in url check, we prevent this from happening. I am assuming, of course, that a class timetable never has overlaps
+    elif "Aulas Sobrepostas" in str(last_table):
         rows = last_table("tr", {"class": "d"})
 
         for row in rows:
@@ -444,12 +483,13 @@ def parse_timetable(credentials: _Credentials.Credentials, url: str) -> list:
             else:
                 url+=weeks_url_query # For some odd reason, single classes' urls don't include the start and finish weeks, a bug on sigarra's side perhaps?
 
-            for event in parse_timetable(credentials, url):
+            for event in parse_timetable(credentials, url, ignore_coherence):
                 if event["weekday"] == weekday and event["start"] == start:
                     result.append(event)
                     break
             else:
-                raise Exception  # Something went wrong
+                if not ignore_coherence:
+                    raise CoherenceError(url, minute, hour, weekday)
 
     def sort_key(event): # Sort by day and then by hour
         return (_weekdays.index(event["weekday"]), event["start"])
